
# =======================================================================
# Wrapper for PerlDevKit/PerlSVC
# Copyright (C) 2005  Thomas Aeby
# All Rights Reserved
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
# 
# =======================================================================
# 
# Author: Thomas Aeby, Thomas Aeby
# E-Mail: aeby@graeff.com
# 
# Thomas Aeby
# Kirchweg 52
# 1735 Giffers
# aeby@graeff.com
# 
# =======================================================================
# 
# @(#) $Id: perlsvc.pl,v 1.2 2005/11/25 10:24:55 aeby Exp $
# 
# Revision History:
# 
# $Log: perlsvc.pl,v $
# Revision 1.2  2005/11/25 10:24:55  aeby
# handling of QUIT signal: only register a handler in BS_unix.pm, do not
# register a handler in the main program of each daemon. In perlsvc.pl
# register a QUIT and a BREAK handler, log receipt and besides that ignore
# the signal (since we get a QUIT or BREAK signal each time a user logs off
# when running as a windows service!)
#
# Revision 1.1  2005/08/09 15:00:20  aeby
# some utilities and files for compiling via ActiveState PDK
#
#  
# =======================================================================

package PerlSvc;

our %Config;
our $started;

our %ProgPars = (
    "uxmon" => {
        "service" => "uxmon",
	"description" => "Big Sister Agent uxmon",
	"startmode" => "auto",
    },
    "bsmon" => {
        "service" => "bsmon",
	"description" => "Big Sister Monitor bsmon",
	"startmode" => "demand",
    }
);
our $progpars;

#open( STDERR, ">\\error.log" );
#open( STDOUT, ">&1" );
#$| = 1;
#select STDERR;
#$| = 1;
#select STDOUT;

my $key = $0;
$key =~ s#.*\\##;
$key =~ s#.exe$##;
unless( $ProgPars{$key} ) {
    print STDERR "called name $key is not known";
}
$progpars = $ProgPars{$key};


sub Startup {
    $SIG{BREAK} = sub {
        BigSister::common::log( "notice", "caught BREAK signal" );
    };
    $SIG{QUIT} = sub {
        BigSister::common::log( "notice", "caught QUIT signal" );
    };
    $started = 1;
    print STDERR "started\n";
    require "main.pl";
}


sub Pause {
}

sub Continue {
}


sub Interactive {
    $started = 1;
    print "starting $progpars->{startfile}\n";
    print "included files:\n";
    while( my( $name, $path ) = each %INC ) {
        print "$name => $path\n";
    }
    eval {
	require "main.pl";
    };
    if( $@ ) {
        print STDERR "FAILED: $@\n";
    }
}


sub wait_service_startup {
    while( ! $started ) {
        print STDERR "waiting for startup\n";
	PerlSvc::ContinueRun( 1 );
    }
}

sub poll {
    my( $time ) = @_;

    if( PerlSvc::RunningAsService() ) {
        PerlSvc::ContinueRun( $time ) || exit(0);
    }
    else {
        return unless defined( $time );
	sleep( $time );
    }
}


####################################

sub Install {
    $Config{"ServiceName"} = $progpars->{"service"};
    $Config{"DisplayName"} = $progpars->{"description"};
    $Config{"StartType"} = $progpars->{"startmode"};
    $Config{"Interactive"} = 0;
}

sub Remove {
    $Config{"ServiceName"} = $progpars->{"service"};
}
