# MySQL monitor
#
# Gert Dewit <gert.dewit@sos.be>
#
# $Id: mysql.pm,v 1.1 2004/06/22 07:10:56 gertdewit Exp $
#
# Check the README file for more information, after installation it should be in
# the $BSHOME/moduleinfo/ directory

package Requester::mysql;

use strict;

@Requester::mysql::ISA = qw(Requester::Requester);

use DBI;

my $mysql = new Requester::mysql();

sub new
{
    my ( $this, $args ) = @_;
    my  $class = ref($this) || $this;
    my  $self = {};
    bless( $self, $class );
    $self->register("mysql");
    return( $self );
}

sub getinfo {
    my( $self, $vars, $indexes ) = @_;

    return undef unless( $indexes );
    my $info = {};

    my $env = $self->{"env"};
    my $host = $self->{"env"}->{"nodeinstance"}->{"host"};
    my $user = $env->expand_value( "user" );
    my $pass = $env->expand_value( "password" );
    my $warn = $env->expand_value( "warn" );
    my $fail = $env->expand_value( "fail" );
    $user = "test" unless ( $user );
    $pass = "" unless ( $pass );
    $warn = 100 unless( $warn );
    $fail = 150 unless( defined($fail) );

    foreach my $test (@$vars) {
	unless( $test =~ /^(threads|status|message)\.(\d+)$/ ) {
	   next;
	}
	my $id = $2;
	next if( defined $info->{$id} );
	$info->{$id} = {};
        my $dsn = sprintf("DBI:mysql:host=%s;mysql_connect_timeout=10", $host); 
#DEBUG BEGIN
#BigSister::common::log( "notice", "mysql : dsn =$dsn : fail = $fail");
#DEBUG END
        my $dbh = DBI->connect($dsn, $user, $pass, { PrintError => 0 });
	if($dbh) {
	  $info->{$id}->{"message"} = "version : ".$dbh->{'mysql_serverinfo'}." ".$dbh->{'mysql_stat'};
	  if($fail > 0) {
            my $q = $dbh->prepare("SHOW status like 'threads_connected'");
            if($q->execute()) {
	      if(my $ref = $q->fetchrow_arrayref()) {
	        $info->{$id}->{"threads"} = $$ref[1];
                if($info->{$id}->{"threads"} > $fail) {
	          $info->{$id}->{"status"} = 0;
		  $info->{$id}->{"message"} .= "\nFAIL : threads > ".$fail;
	        } elsif ($info->{$id}->{"threads"} > $warn) {
	          $info->{$id}->{"status"} = 1;
		  $info->{$id}->{"message"} .= "\nFAIL : threads > ".$warn;
	        } else {
	          $info->{$id}->{"status"} = 3;
	  	  $info->{$id}->{"message"} .= "\nthreads OK";
	        }
	      } else {
	        $info->{$id}->{"message"} .= "\nERROR : NO PERFORMANCE DATA";
	        $info->{$id}->{"threads"} = -1;
	        $info->{$id}->{"status"} = 1;
	      }
	      $q->finish();
	    } else {
	      $info->{$id}->{"message"} .= "\nERROR : ".$q->errstr;
	      $info->{$id}->{"threads"} = -1;
	      $info->{$id}->{"status"} = 1;
	    }
	  } else {
	    $info->{$id}->{"message"} .= "\nConnection OK";
	    $info->{$id}->{"threads"} = -1;
	    $info->{$id}->{"status"} = 3;
	  }
	  $dbh->disconnect();
	} else {
	  $info->{$id}->{"message"} = "Cannot connect : ".$DBI::errstr;
	  $info->{$id}->{"threads"} = -1;
	  $info->{$id}->{"status"} = 0;
	}
    }
    return $info;
}


1;
